# Lovre Bogdanic, 19DEC2022
# Modified by Tetsuya Miwa, 10JAN2023
# PyQtGraph code based on Kurene's https://www.wizard-notes.com/entry/python/pyqtgraph-realtime-2d-scatter-plot

import ctcsound as csound
from pyqtgraph.Qt import QtGui, QtCore
import numpy as np
import pyqtgraph as pg
import numpy as np


fftSize = 1024 # this needs to match gifftsiz in orc
fs = 48000 # sample rate

orc = '''
sr = 48000
0dbfs = 1
nchnls = 2

;general values for fourier transform
gifftsiz  =         1024   ; this needs to match above defined fftSize
gioverlap =         256
giwintyp  =         1 ;von hann window

;an 'empty' function table
giTableAmp  ftgen   1, 0, -(gifftsiz/2+1), 2, 0  ;number of analysis bin = gifftsiz/2 +1
giTableFreq ftgen   2, 0, -(gifftsiz/2+1), 2, 0  ;number of analysis bin = gifftsiz/2 +1

instr 1
    ;aout	oscili 0.5, 500, 1
    aout = oscili(0.5,line(100,p3,8000),1)
	aout	*= linseg:k(0, 1, 1, p3-1, 0)

    outs aout,aout

    kArr[] init gifftsiz+2 ;create array for bin data

    fsig	pvsanal aout, gifftsiz, gioverlap, gifftsiz, giwintyp
    kflag pvsftw fsig, giTableAmp, giTableFreq
endin
'''
sco = '''
f1 0 1024 7 1 512 1 0 -1 512 -1\n
i1 1 10\n
'''

cs = csound.Csound()
cs.setOption('-odac')
cs.compileOrc(orc)
cs.readScore(sco)
cs.start()

pt = csound.CsoundPerformanceThread(cs.csound())
pt.play()

app = QtGui.QApplication([])
win = pg.GraphicsLayoutWidget()
win.show()

p = win.addPlot()
mypen = pg.mkPen(16,255,255, width=5)
line = p.plot(pen = mypen)
pg.setConfigOptions(antialias=True)

f_axis_delta = fs/fftSize # fundamental of analysis
f_axis = np.arange(0,fs/2 + f_axis_delta,f_axis_delta) # frequency bins
fftArrayAmp  = np.zeros(int(fftSize/2 + 1)) # array length must be fftSize/2 +1
fftArrayFreq = np.zeros(int(fftSize/2 + 1)) # array length must be fftSize/2 +1

p.setXRange(0,fs/2)
p.setYRange(0,1)

def animate():
    global fftArrayAmp, f_axis_cs, f_axis, line, plot
    cs.tableCopyOut(1,fftArrayAmp)
    cs.tableCopyOut(2,fftArrayFreq)
    f_axis_cs = fftArrayFreq # estimated frequency by phase vocoder
    amp_vals = fftArrayAmp # fft amplitude values
    line.setData(f_axis, amp_vals) # Use frequency bins for x axis value
    #line.setData(f_axis_cs, amp_vals) # Use estimated frequency for x axis value

timer = QtCore.QTimer()
timer.timeout.connect(animate)
timer.start(5) #Update every 5 msec

if __name__ == '__main__':
    import sys
    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
        QtGui.QApplication.instance().exec_()